@@ -11,7 +11,7 @@ export PGPORT=55432
 export PGDATA=$PGPATH/data_${PGPORT}
 export PGLOG="$PGDATA/pgsql.log"
 # What to use to start up the postmaster
-DAEMON="$PGPATH/bin/pg_ctl -D $PGDATA -l logfile start"
+DAEMON="$PGPATH/bin/pg_ctl -D $PGDATA -o '-F' -l logfile start"
 
 # What to use to shut down the postmaster
 PGCTL="$PGPATH/bin/pg_ctl"
