@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/usr/bin/env bash
 export PG_VER=9.6
 # export PGPORT=8442
 export OS_BUILD=64
@@ -44,7 +44,7 @@ if test -f utils/repo_revision.pl; then
 	perl utils/repo_revision.pl
 fi
 export VREV="`cat postgis_revision.h | awk '{print $3}'`"
-echo "GIT is ${VREV}"
+echo "SVN is ${VREV}"
 cd doc
 
 
@@ -70,7 +70,7 @@ cp html/images/* images
 make epub
 make -e chunked-html-web 2>&1 | tee -a doc-errors.log
 
-if [[ "$reference" == *"trunk"* ]]; then  #only do this for trunk because only trunk follows transifex
+if [[ "$reference" == *"master"* ]]; then  #only do this for trunk because only trunk follows transifex
   make update-pot
   make pull-tx
   make -C po/it_IT/ local-html
