@@ -1032,7 +1032,7 @@ ptarray_isccw(const POINTARRAY *pa)
 }
 
 POINTARRAY*
-ptarray_force_dims(const POINTARRAY *pa, int hasz, int hasm)
+ptarray_force_dims(const POINTARRAY *pa, int hasz, int hasm, double zval, double mval)
 {
 	/* TODO handle zero-length point arrays */
 	uint32_t i;
@@ -1045,9 +1045,9 @@ ptarray_force_dims(const POINTARRAY *pa, int hasz, int hasm)
 	{
 		getPoint4d_p(pa, i, &pt);
 		if( hasz && ! in_hasz )
-			pt.z = 0.0;
+			pt.z = zval;
 		if( hasm && ! in_hasm )
-			pt.m = 0.0;
+			pt.m = mval;
 		ptarray_append_point(pa_out, &pt, LW_TRUE);
 	}
 
@@ -1500,14 +1500,14 @@ ptarray_remove_repeated_points_in_place(POINTARRAY *pa, double tolerance, uint32
 			if (last_point && n_points_out > 1 && tolerance > 0.0 && dsq <= tolsq)
 			{
 				n_points_out--;
-				p_to -= pt_size;
+				p_to = (char*)p_to - pt_size;
 			}
 		}
 
 		/* Compact all remaining values to front of array */
 		memcpy(p_to, pt, pt_size);
 		n_points_out++;
-		p_to += pt_size;
+		p_to = (char*)p_to + pt_size;
 		last = pt;
 	}
 	/* Adjust array length */
@@ -1583,6 +1583,49 @@ ptarray_dp_findsplit_in_place(const POINTARRAY *pts, uint32_t it_first, uint32_t
 	return split;
 }
 
+/* O(N) simplification for tolearnce = 0 */
+static void
+ptarray_simplify_in_place_tolerance0(POINTARRAY *pa)
+{
+	uint32_t kept_it = 0;
+	uint32_t last_it = pa->npoints - 1;
+	const POINT2D *kept_pt = getPoint2d_cp(pa, 0);
+	const size_t pt_size = ptarray_point_size(pa);
+
+	for (uint32_t i = 1; i < last_it; i++)
+	{
+		const POINT2D *curr_pt = getPoint2d_cp(pa, i);
+		const POINT2D *next_pt = getPoint2d_cp(pa, i + 1);
+
+		double ba_x = next_pt->x - kept_pt->x;
+		double ba_y = next_pt->y - kept_pt->y;
+		double ab_length_sqr = ba_x * ba_x + ba_y * ba_y;
+
+		double ca_x = curr_pt->x - kept_pt->x;
+		double ca_y = curr_pt->y - kept_pt->y;
+		double dot_ac_ab = ca_x * ba_x + ca_y * ba_y;
+		double s_numerator = ca_x * ba_y - ca_y * ba_x;
+
+		if (dot_ac_ab < 0.0 || dot_ac_ab > ab_length_sqr || s_numerator != 0)
+		{
+			kept_it++;
+			kept_pt = curr_pt;
+			if (kept_it != i)
+				memcpy(pa->serialized_pointlist + pt_size * kept_it,
+				       pa->serialized_pointlist + pt_size * i,
+				       pt_size);
+		}
+	}
+
+	/* Append last point */
+	kept_it++;
+	if (kept_it != last_it)
+		memcpy(pa->serialized_pointlist + pt_size * kept_it,
+		       pa->serialized_pointlist + pt_size * last_it,
+		       pt_size);
+	pa->npoints = kept_it + 1;
+}
+
 void
 ptarray_simplify_in_place(POINTARRAY *pa, double tolerance, uint32_t minpts)
 {
@@ -1590,6 +1633,12 @@ ptarray_simplify_in_place(POINTARRAY *pa, double tolerance, uint32_t minpts)
 	if (pa->npoints < 3 || pa->npoints <= minpts)
 		return;
 
+	if (tolerance == 0 && minpts <= 2)
+	{
+		ptarray_simplify_in_place_tolerance0(pa);
+		return;
+	}
+
 	/* We use this array to keep track of the points we are keeping, so
 	 * we store just TRUE / FALSE in their position */
 	uint8_t *kept_points = lwalloc(sizeof(uint8_t) * pa->npoints);
@@ -1643,7 +1692,7 @@ ptarray_simplify_in_place(POINTARRAY *pa, double tolerance, uint32_t minpts)
 		       pa->serialized_pointlist + pt_size * (pa->npoints - 1),
 		       pt_size);
 	}
-	else
+	else if (pa->npoints != keptn) /* We don't need to move any points if we are keeping them all */
 	{
 		for (uint32_t i = 1; i < pa->npoints; i++)
 		{
@@ -1757,49 +1806,30 @@ ptarray_length(const POINTARRAY *pts)
 void
 ptarray_affine(POINTARRAY *pa, const AFFINE *a)
 {
-	uint32_t i;
-	double x,y,z;
-	POINT4D p4d;
-
-	LWDEBUG(2, "lwgeom_affine_ptarray start");
-
-	if ( FLAGS_GET_Z(pa->flags) )
+	if (FLAGS_GET_Z(pa->flags))
 	{
-		LWDEBUG(3, " has z");
-
-		for (i=0; i<pa->npoints; i++)
+		for (uint32_t i = 0; i < pa->npoints; i++)
 		{
-			getPoint4d_p(pa, i, &p4d);
-			x = p4d.x;
-			y = p4d.y;
-			z = p4d.z;
-			p4d.x = a->afac * x + a->bfac * y + a->cfac * z + a->xoff;
-			p4d.y = a->dfac * x + a->efac * y + a->ffac * z + a->yoff;
-			p4d.z = a->gfac * x + a->hfac * y + a->ifac * z + a->zoff;
-			ptarray_set_point4d(pa, i, &p4d);
-
-			LWDEBUGF(3, " POINT %g %g %g => %g %g %g", x, y, z, p4d.x, p4d.y, p4d.z);
+			POINT4D *p4d = (POINT4D *)(getPoint_internal(pa, i));
+			double x = p4d->x;
+			double y = p4d->y;
+			double z = p4d->z;
+			p4d->x = a->afac * x + a->bfac * y + a->cfac * z + a->xoff;
+			p4d->y = a->dfac * x + a->efac * y + a->ffac * z + a->yoff;
+			p4d->z = a->gfac * x + a->hfac * y + a->ifac * z + a->zoff;
 		}
 	}
 	else
 	{
-		LWDEBUG(3, " doesn't have z");
-
-		for (i=0; i<pa->npoints; i++)
+		for (uint32_t i = 0; i < pa->npoints; i++)
 		{
-			getPoint4d_p(pa, i, &p4d);
-			x = p4d.x;
-			y = p4d.y;
-			p4d.x = a->afac * x + a->bfac * y + a->xoff;
-			p4d.y = a->dfac * x + a->efac * y + a->yoff;
-			ptarray_set_point4d(pa, i, &p4d);
-
-			LWDEBUGF(3, " POINT %g %g => %g %g", x, y, p4d.x, p4d.y);
+			POINT2D *pt = (POINT2D *)(getPoint_internal(pa, i));
+			double x = pt->x;
+			double y = pt->y;
+			pt->x = a->afac * x + a->bfac * y + a->xoff;
+			pt->y = a->dfac * x + a->efac * y + a->yoff;
 		}
 	}
-
-	LWDEBUG(3, "lwgeom_affine_ptarray end");
-
 }
 
 /**
@@ -1977,63 +2007,58 @@ ptarray_startpoint(const POINTARRAY *pa, POINT4D *pt)
 void
 ptarray_grid_in_place(POINTARRAY *pa, const gridspec *grid)
 {
-	uint32_t i, j = 0;
+	uint32_t j = 0;
 	POINT4D *p, *p_out = NULL;
-	int ndims = FLAGS_NDIMS(pa->flags);
-	int has_z = FLAGS_GET_Z(pa->flags);
-	int has_m = FLAGS_GET_M(pa->flags);
+	double x, y, z = 0, m = 0;
+	uint32_t ndims = FLAGS_NDIMS(pa->flags);
+	uint32_t has_z = FLAGS_GET_Z(pa->flags);
+	uint32_t has_m = FLAGS_GET_M(pa->flags);
 
-	LWDEBUGF(2, "%s called on %p", __func__, pa);
-
-	for (i = 0; i < pa->npoints; i++)
+	for (uint32_t i = 0; i < pa->npoints; i++)
 	{
 		/* Look straight into the abyss */
-		p = (POINT4D*)(getPoint_internal(pa, i));
+		p = (POINT4D *)(getPoint_internal(pa, i));
+		x = p->x;
+		y = p->y;
+		if (ndims > 2)
+			z = p->z;
+		if (ndims > 3)
+			m = p->m;
 
 		if (grid->xsize > 0)
-		{
-			p->x = rint((p->x - grid->ipx)/grid->xsize) * grid->xsize + grid->ipx;
-		}
+			x = rint((x - grid->ipx) / grid->xsize) * grid->xsize + grid->ipx;
 
 		if (grid->ysize > 0)
-		{
-			p->y = rint((p->y - grid->ipy)/grid->ysize) * grid->ysize + grid->ipy;
-		}
+			y = rint((y - grid->ipy) / grid->ysize) * grid->ysize + grid->ipy;
 
 		/* Read and round this point */
 		/* Z is always in third position */
-		if (has_z)
-		{
-			if (grid->zsize > 0)
-				p->z = rint((p->z - grid->ipz)/grid->zsize) * grid->zsize + grid->ipz;
-		}
+		if (has_z && grid->zsize > 0)
+			z = rint((z - grid->ipz) / grid->zsize) * grid->zsize + grid->ipz;
+
 		/* M might be in 3rd or 4th position */
-		if (has_m)
+		if (has_m && grid->msize > 0)
 		{
-			/* In POINT M, M is in 3rd position */
-			if (grid->msize > 0 && !has_z)
-				p->z = rint((p->z - grid->ipm)/grid->msize) * grid->msize + grid->ipm;
-			/* In POINT ZM, M is in 4th position */
-			if (grid->msize > 0 && has_z)
-				p->m = rint((p->m - grid->ipm)/grid->msize) * grid->msize + grid->ipm;
+			/* In POINT ZM, M is in 4th position, in POINT M, M is in 3rd position which is Z in POINT4D */
+			if (has_z)
+				m = rint((m - grid->ipm) / grid->msize) * grid->msize + grid->ipm;
+			else
+				z = rint((z - grid->ipm) / grid->msize) * grid->msize + grid->ipm;
 		}
 
 		/* Skip duplicates */
-		if ( p_out && FP_EQUALS(p_out->x, p->x) && FP_EQUALS(p_out->y, p->y)
-		   && (ndims > 2 ? FP_EQUALS(p_out->z, p->z) : 1)
-		   && (ndims > 3 ? FP_EQUALS(p_out->m, p->m) : 1) )
-		{
+		if (p_out && p_out->x == x && p_out->y == y && (ndims > 2 ? p_out->z == z : 1) &&
+		    (ndims > 3 ? p_out->m == m : 1))
 			continue;
-		}
 
 		/* Write rounded values into the next available point */
-		p_out = (POINT4D*)(getPoint_internal(pa, j++));
-		p_out->x = p->x;
-		p_out->y = p->y;
+		p_out = (POINT4D *)(getPoint_internal(pa, j++));
+		p_out->x = x;
+		p_out->y = y;
 		if (ndims > 2)
-			p_out->z = p->z;
+			p_out->z = z;
 		if (ndims > 3)
-			p_out->m = p->m;
+			p_out->m = m;
 	}
 
 	/* Update output ptarray length */
@@ -2041,7 +2066,6 @@ ptarray_grid_in_place(POINTARRAY *pa, const gridspec *grid)
 	return;
 }
 
-
 int
 ptarray_npoints_in_rect(const POINTARRAY *pa, const GBOX *gbox)
 {
