@@ -71,7 +71,7 @@ Oid postgis_oid(postgisType typ);
  * Note that it's length can be up to strlen(schema) + "." + strlen("spatial_ref_sys") + NULL, i.e: 80 bytes
  * Only useful if postgis_initialize_cache has been called before. Otherwise returns "spatial_ref_sys"
  */
-const char *postgis_spatial_ref_sys(void);
+const char *postgis_spatial_ref_sys();
 
 /****************************************************************************************/
 
@@ -89,7 +89,12 @@ void pg_install_lwgeom_handlers(void);
 /* Argument handling macros */
 #define PG_GETARG_GSERIALIZED_P(varno) ((GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(varno)))
 #define PG_GETARG_GSERIALIZED_P_COPY(varno) ((GSERIALIZED *)PG_DETOAST_DATUM_COPY(PG_GETARG_DATUM(varno)))
-#define PG_GETARG_GSERIALIZED_P_SLICE(varno, start, size) ((GSERIALIZED *)PG_DETOAST_DATUM_SLICE(PG_GETARG_DATUM(varno), start, size))
+#define PG_GSERIALIZED_DATUM_NEEDS_DETOAST(datum) \
+	(VARATT_IS_EXTENDED((datum)) || VARATT_IS_EXTERNAL((datum)) || VARATT_IS_COMPRESSED((datum)))
+#define PG_GETARG_GSERIALIZED_HEADER(varno) \
+	PG_GSERIALIZED_DATUM_NEEDS_DETOAST(PG_GETARG_DATUM(varno)) \
+	? ((GSERIALIZED *)PG_DETOAST_DATUM_SLICE(PG_GETARG_DATUM(varno), 0, gserialized_max_header_size())) \
+	: ((GSERIALIZED *)(PG_GETARG_DATUM(varno)))
 
 /* Debugging macros */
 #if POSTGIS_DEBUG_LEVEL > 0
@@ -183,6 +188,7 @@ void gserialized_error_if_srid_mismatch_reference(const GSERIALIZED *g1, const i
 * Fails on empty.
 */
 int gserialized_datum_get_gbox_p(Datum gsdatum, GBOX *gbox);
+int gserialized_datum_get_internals_p(Datum gsdatum, GBOX *gbox, lwflags_t *flags, uint8_t *type, int32_t *srid);
 
 /* PG-exposed */
 Datum BOX2D_same(PG_FUNCTION_ARGS);
