@@ -32,8 +32,6 @@ char *GetProj4String(int32_t srid);
  */
 typedef void *ProjCache ;
 
-bool IsInPROJCache(ProjCache cache, int32_t srid_from, int32_t srid_to);
-PJ *GetPJFromPROJCache(ProjCache cache, int32_t srid_from, int32_t srid_to);
 int GetPJUsingFCInfo(FunctionCallInfo fcinfo, int32_t srid_from, int32_t srid_to, LWPROJ **pj);
 int spheroid_init_from_srid(FunctionCallInfo fcinfo, int32_t srid, SPHEROID *s);
 void srid_check_latlong(FunctionCallInfo fcinfo, int32_t srid);
