@@ -46,6 +46,7 @@
 #include "../postgis_config.h"
 
 #include "liblwgeom.h"         /* For standard geometry types. */
+#include "liblwgeom_internal.h"
 #include "lwgeom_pg.h"       /* For debugging macros. */
 #include "gserialized_gist.h"	     /* For utility functions. */
 
@@ -116,14 +117,24 @@ typedef bool (*box2df_predicate)(const BOX2DF *a, const BOX2DF *b);
 
 static char* box2df_to_string(const BOX2DF *a)
 {
-	char *rv = NULL;
+	static const int precision = 12;
+	char tmp[13 + 4 * OUT_MAX_BYTES_DOUBLE] = {'B', 'O', 'X', '2', 'D', 'F', '(', 0};
+	int len = 7;
 
-	if ( a == NULL )
+	if (a == NULL)
 		return pstrdup("<NULLPTR>");
 
-	rv = palloc(128);
-	sprintf(rv, "BOX2DF(%.12g %.12g, %.12g %.12g)", a->xmin, a->ymin, a->xmax, a->ymax);
-	return rv;
+	len += lwprint_double(a->xmin, precision, &tmp[len]);
+	tmp[len++] = ' ';
+	len += lwprint_double(a->ymin, precision, &tmp[len]);
+	tmp[len++] = ',';
+	tmp[len++] = ' ';
+	len += lwprint_double(a->xmax, precision, &tmp[len]);
+	tmp[len++] = ' ';
+	len += lwprint_double(a->ymax, precision, &tmp[len]);
+	tmp[len++] = ')';
+
+	return pstrdup(tmp);
 }
 
 /* Allocate a new copy of BOX2DF */
@@ -439,70 +450,106 @@ static double box2df_distance(const BOX2DF *a, const BOX2DF *b)
 	return FLT_MAX;
 }
 
+/**
+ * Peak into a #GSERIALIZED datum to find its bounding box and some other metadata. If the box is there, copy it out and
+ * return it. If not, calculate the box from the full object and return the box based on that. If no box is available,
+ * return #LW_FAILURE, otherwise #LW_SUCCESS.
+ */
+int
+gserialized_datum_get_internals_p(Datum gsdatum, GBOX *gbox, lwflags_t *flags, uint8_t *type, int32_t *srid)
+{
+	int result = LW_SUCCESS;
+	GSERIALIZED *gpart = NULL;
+	int need_detoast = PG_GSERIALIZED_DATUM_NEEDS_DETOAST((struct varlena *)gsdatum);
+	if (need_detoast)
+	{
+		gpart = (GSERIALIZED *)PG_DETOAST_DATUM_SLICE(gsdatum, 0, gserialized_max_header_size());
+	}
+	else
+	{
+		gpart = (GSERIALIZED *)gsdatum;
+	}
+
+	if (!gserialized_has_bbox(gpart) && need_detoast && LWSIZE_GET(gpart->size) >= gserialized_max_header_size())
+	{
+		/* The headers don't contain a bbox and the object is larger than what we retrieved, so
+		 * we now detoast it completely */
+		POSTGIS_FREE_IF_COPY_P(gpart, gsdatum);
+		gpart = (GSERIALIZED *)PG_DETOAST_DATUM(gsdatum);
+	}
+
+	result = gserialized_get_gbox_p(gpart, gbox);
+	*flags = gserialized_get_lwflags(gpart);
+	*srid = gserialized_get_srid(gpart);
+	*type = gserialized_get_type(gpart);
+
+	POSTGIS_FREE_IF_COPY_P(gpart, gsdatum);
+	return result;
+}
 
 /**
-* Peak into a #GSERIALIZED datum to find the bounding box. If the
-* box is there, copy it out and return it. If not, calculate the box from the
-* full object and return the box based on that. If no box is available,
-* return #LW_FAILURE, otherwise #LW_SUCCESS.
-*/
+ * Given a #GSERIALIZED datum, as quickly as possible (peaking into the top
+ * of the memory) return the gbox extents. Does not deserialize the geometry,
+ * but <em>WARNING</em> returns a slightly larger bounding box than actually
+ * encompasses the objects. For geography objects returns geocentric bounding
+ * box, for geometry objects returns cartesian bounding box.
+ */
+int
+gserialized_datum_get_gbox_p(Datum gsdatum, GBOX *gbox)
+{
+	uint8_t type;
+	int32_t srid;
+	lwflags_t flags;
+
+	return gserialized_datum_get_internals_p(gsdatum, gbox, &flags, &type, &srid);
+}
+
+/* Note that this duplicates code from gserialized_datum_get_internals_p. It does so because
+ * accessing only the BOX2DF (as floats) is faster and this is a hot path function in indexes
+ */
 int
 gserialized_datum_get_box2df_p(Datum gsdatum, BOX2DF *box2df)
 {
-	GSERIALIZED *gpart;
 	int result = LW_SUCCESS;
+	GSERIALIZED *gpart = NULL;
+	int need_detoast = PG_GSERIALIZED_DATUM_NEEDS_DETOAST((struct varlena *)gsdatum);
+	if (need_detoast)
+	{
+		gpart = (GSERIALIZED *)PG_DETOAST_DATUM_SLICE(gsdatum, 0, gserialized_max_header_size());
+	}
+	else
+	{
+		gpart = (GSERIALIZED *)gsdatum;
+	}
 
-	POSTGIS_DEBUG(4, "entered function");
-
-	/*
-	** Because geometry is declared as "storage = main" anything large
-	** enough to take serious advantage of PG_DETOAST_DATUM_SLICE will have
-	** already been compressed, which means the entire object will be
-	** fetched and decompressed before a slice is taken, thus removing
-	** any efficiencies gained from slicing.
-	** As of Pg12 we can partially decompress a toasted object
-	** (though we still need to fully retrieve it from TOAST)
-	** which makes slicing worthwhile.
-	*/
-	gpart = (GSERIALIZED*)PG_DETOAST_DATUM(gsdatum);
-
-	POSTGIS_DEBUGF(4, "got flags %d", gpart->gflags);
-
-	/* Do we even have a serialized bounding box? */
 	if (gserialized_has_bbox(gpart))
 	{
-		/* Yes! Copy it out into the box! */
 		size_t box_ndims;
 		const float *f = gserialized_get_float_box_p(gpart, &box_ndims);
-
-		POSTGIS_DEBUG(4, "copying box out of serialization");
 		memcpy(box2df, f, sizeof(BOX2DF));
 		result = LW_SUCCESS;
 	}
 	else
 	{
-		/* No, we need to calculate it from the full object. */
-		GBOX gbox;
-		gbox_init(&gbox);
-
+		GBOX gbox = {0};
+		if (need_detoast && LWSIZE_GET(gpart->size) >= gserialized_max_header_size())
+		{
+			/* The headers don't contain a bbox and the object is larger than what we retrieved, so
+			 * we now detoast it completely and recheck */
+			POSTGIS_FREE_IF_COPY_P(gpart, gsdatum);
+			gpart = (GSERIALIZED *)PG_DETOAST_DATUM(gsdatum);
+		}
 		result = gserialized_get_gbox_p(gpart, &gbox);
-		if ( result == LW_SUCCESS )
+		if (result == LW_SUCCESS)
 		{
 			result = box2df_from_gbox_p(&gbox, box2df);
 		}
-		else
-		{
-			POSTGIS_DEBUG(4, "could not calculate bbox");
-		}
 	}
 
 	POSTGIS_FREE_IF_COPY_P(gpart, gsdatum);
-	POSTGIS_DEBUGF(4, "result = %d, got box2df %s", result, result == LW_SUCCESS ? box2df_to_string(box2df) : "NONE");
-
 	return result;
 }
 
-
 /**
 * Support function. Based on two datums return true if
 * they satisfy the predicate and false otherwise.
