@@ -32,6 +32,7 @@
 
 #include "../postgis_config.h"
 #include "liblwgeom.h"
+#include "liblwgeom_internal.h"
 #include "lwgeom_pg.h"
 
 #include <math.h>
@@ -186,12 +187,6 @@ Datum postgis_lib_version(PG_FUNCTION_ARGS)
 	PG_RETURN_TEXT_P(result);
 }
 
-PG_FUNCTION_INFO_V1(postgis_svn_version);
-Datum postgis_svn_version(PG_FUNCTION_ARGS)
-{
-	return postgis_lib_revision(fcinfo);
-}
-
 PG_FUNCTION_INFO_V1(postgis_lib_revision);
 Datum postgis_lib_revision(PG_FUNCTION_ARGS)
 {
@@ -389,13 +384,14 @@ Datum LWGEOM_force_3dz(PG_FUNCTION_ARGS)
 	GSERIALIZED *pg_geom_in = PG_GETARG_GSERIALIZED_P(0);
 	GSERIALIZED *pg_geom_out;
 	LWGEOM *lwg_in, *lwg_out;
+	double z = PG_NARGS() < 2 ? 0 : PG_GETARG_FLOAT8(1);
 
 	/* already 3d */
 	if (gserialized_ndims(pg_geom_in) == 3 && gserialized_has_z(pg_geom_in))
 		PG_RETURN_POINTER(pg_geom_in);
 
 	lwg_in = lwgeom_from_gserialized(pg_geom_in);
-	lwg_out = lwgeom_force_3dz(lwg_in);
+	lwg_out = lwgeom_force_3dz(lwg_in, z);
 	pg_geom_out = geometry_serialize(lwg_out);
 	lwgeom_free(lwg_out);
 	lwgeom_free(lwg_in);
@@ -411,13 +407,14 @@ Datum LWGEOM_force_3dm(PG_FUNCTION_ARGS)
 	GSERIALIZED *pg_geom_in = PG_GETARG_GSERIALIZED_P(0);
 	GSERIALIZED *pg_geom_out;
 	LWGEOM *lwg_in, *lwg_out;
+	double m = PG_NARGS() < 2 ? 0 : PG_GETARG_FLOAT8(1);
 
 	/* already 3d */
 	if (gserialized_ndims(pg_geom_in) == 3 && gserialized_has_m(pg_geom_in))
 		PG_RETURN_POINTER(pg_geom_in);
 
 	lwg_in = lwgeom_from_gserialized(pg_geom_in);
-	lwg_out = lwgeom_force_3dm(lwg_in);
+	lwg_out = lwgeom_force_3dm(lwg_in, m);
 	pg_geom_out = geometry_serialize(lwg_out);
 	lwgeom_free(lwg_out);
 	lwgeom_free(lwg_in);
@@ -433,13 +430,15 @@ Datum LWGEOM_force_4d(PG_FUNCTION_ARGS)
 	GSERIALIZED *pg_geom_in = PG_GETARG_GSERIALIZED_P(0);
 	GSERIALIZED *pg_geom_out;
 	LWGEOM *lwg_in, *lwg_out;
+	double z = PG_NARGS() < 3 ? 0 : PG_GETARG_FLOAT8(1);
+	double m = PG_NARGS() < 3 ? 0 : PG_GETARG_FLOAT8(2);
 
 	/* already 4d */
 	if (gserialized_ndims(pg_geom_in) == 4)
 		PG_RETURN_POINTER(pg_geom_in);
 
 	lwg_in = lwgeom_from_gserialized(pg_geom_in);
-	lwg_out = lwgeom_force_4d(lwg_in);
+	lwg_out = lwgeom_force_4d(lwg_in, z, m);
 	pg_geom_out = geometry_serialize(lwg_out);
 	lwgeom_free(lwg_out);
 	lwgeom_free(lwg_in);
@@ -1912,8 +1911,6 @@ Datum LWGEOM_noop(PG_FUNCTION_ARGS)
 	GSERIALIZED *in = PG_GETARG_GSERIALIZED_P(0);
 	LWGEOM *lwgeom = lwgeom_from_gserialized(in);
 	GSERIALIZED *out = geometry_serialize(lwgeom);
-	lwgeom_free(lwgeom);
-	PG_FREE_IF_COPY(in, 0);
 	PG_RETURN_POINTER(out);
 }
 
@@ -1953,7 +1950,7 @@ Datum ST_Normalize(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(LWGEOM_zmflag);
 Datum LWGEOM_zmflag(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *in = PG_GETARG_GSERIALIZED_P(0);
+	GSERIALIZED *in = PG_GETARG_GSERIALIZED_HEADER(0);
 	int ret = 0;
 
 	if (gserialized_has_z(in))
@@ -1967,21 +1964,21 @@ Datum LWGEOM_zmflag(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(LWGEOM_hasz);
 Datum LWGEOM_hasz(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *in = PG_GETARG_GSERIALIZED_P(0);
+	GSERIALIZED *in = PG_GETARG_GSERIALIZED_HEADER(0);
 	PG_RETURN_BOOL(gserialized_has_z(in));
 }
 
 PG_FUNCTION_INFO_V1(LWGEOM_hasm);
 Datum LWGEOM_hasm(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *in = PG_GETARG_GSERIALIZED_P(0);
+	GSERIALIZED *in = PG_GETARG_GSERIALIZED_HEADER(0);
 	PG_RETURN_BOOL(gserialized_has_m(in));
 }
 
 PG_FUNCTION_INFO_V1(LWGEOM_hasBBOX);
 Datum LWGEOM_hasBBOX(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *in = PG_GETARG_GSERIALIZED_P(0);
+	GSERIALIZED *in = PG_GETARG_GSERIALIZED_HEADER(0);
 	char res = gserialized_has_bbox(in);
 	PG_FREE_IF_COPY(in, 0);
 	PG_RETURN_BOOL(res);
@@ -1991,7 +1988,7 @@ Datum LWGEOM_hasBBOX(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(LWGEOM_ndims);
 Datum LWGEOM_ndims(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *in = PG_GETARG_GSERIALIZED_P(0);
+	GSERIALIZED *in = PG_GETARG_GSERIALIZED_HEADER(0);
 	int ret = gserialized_ndims(in);
 	PG_FREE_IF_COPY(in, 0);
 	PG_RETURN_INT16(ret);
@@ -2045,6 +2042,7 @@ Datum ST_TileEnvelope(PG_FUNCTION_ARGS)
 	double tileGeoSizeX, tileGeoSizeY;
 	double boundsWidth, boundsHeight;
 	double x1, y1, x2, y2;
+	double margin;
 	/* This is broken, since 3857 doesn't mean "web mercator", it means
 	   the contents of the row in spatial_ref_sys with srid = 3857.
 	   For practical purposes this will work, but in good implementation
@@ -2073,6 +2071,12 @@ Datum ST_TileEnvelope(PG_FUNCTION_ARGS)
 	srid = g->srid;
 	lwgeom_free(g);
 
+	/* Avoid crashing with old signature (old sql code with 3 args, new C code with 4) */
+	margin = PG_NARGS() < 4 ? 0 : PG_GETARG_FLOAT8(4);
+	/* shrinking by more than 50% would eliminate the tile outright */
+	if (margin < -0.5)
+		elog(ERROR, "%s: Margin must not be less than -50%%, margin=%f", __func__, margin);
+
 	boundsWidth  = bbox.xmax - bbox.xmin;
 	boundsHeight = bbox.ymax - bbox.ymin;
 	if (boundsWidth <= 0 || boundsHeight <= 0)
@@ -2091,10 +2095,29 @@ Datum ST_TileEnvelope(PG_FUNCTION_ARGS)
 
 	tileGeoSizeX = boundsWidth / worldTileSize;
 	tileGeoSizeY = boundsHeight / worldTileSize;
-	x1 = bbox.xmin + tileGeoSizeX * (x);
-	x2 = bbox.xmin + tileGeoSizeX * (x+1);
-	y1 = bbox.ymax - tileGeoSizeY * (y+1);
-	y2 = bbox.ymax - tileGeoSizeY * (y);
+
+	/*
+	 * 1 margin (100%) is the same as a single tile width
+	 * if the size of the tile with margins span more than the total number of tiles,
+	 * reset x1/x2 to the bounds
+	 */
+	if ((1 + margin * 2) > worldTileSize)
+	{
+		x1 = bbox.xmin;
+		x2 = bbox.xmax;
+	}
+	else
+	{
+		x1 = bbox.xmin + tileGeoSizeX * (x - margin);
+		x2 = bbox.xmin + tileGeoSizeX * (x + 1 + margin);
+	}
+
+	y1 = bbox.ymax - tileGeoSizeY * (y + 1 + margin);
+	y2 = bbox.ymax - tileGeoSizeY * (y - margin);
+
+	/* Clip y-axis to the given bounds */
+	if (y1 < bbox.ymin) y1 = bbox.ymin;
+	if (y2 > bbox.ymax) y2 = bbox.ymax;
 
 	PG_RETURN_POINTER(
 		geometry_serialize(
@@ -2107,17 +2130,8 @@ Datum ST_TileEnvelope(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(ST_IsCollection);
 Datum ST_IsCollection(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *geom;
-	int type;
-	size_t size;
-
-	/* Pull only a small amount of the tuple, enough to get the type. */
-	/* header + srid/flags + bbox? + type number */
-	size = VARHDRSZ + 8 + 32 + 4;
-
-	geom = PG_GETARG_GSERIALIZED_P_SLICE(0, 0, size);
-
-	type = gserialized_get_type(geom);
+	GSERIALIZED *geom = PG_GETARG_GSERIALIZED_HEADER(0);
+	int type = gserialized_get_type(geom);
 	PG_RETURN_BOOL(lwtype_is_collection(type));
 }
 
@@ -2216,9 +2230,13 @@ Datum LWGEOM_addpoint(PG_FUNCTION_ARGS)
 		}
 		else if (where < 0 || where > (int32)line->points->npoints)
 		{
-			elog(ERROR, "Invalid offset");
+			elog(ERROR, "%s: Invalid offset", __func__);
 			PG_RETURN_NULL();
 		}
+		else
+		{
+			uwhere = where;
+		}
 	}
 
 	point = lwgeom_as_lwpoint(lwgeom_from_gserialized(pglwg2));
@@ -2349,28 +2367,14 @@ Datum LWGEOM_setpoint_linestring(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(LWGEOM_asEWKT);
 Datum LWGEOM_asEWKT(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *geom;
-	LWGEOM *lwgeom;
-	char *wkt;
-	size_t wkt_size;
-	text *result;
+	GSERIALIZED *geom = PG_GETARG_GSERIALIZED_P(0);
+	LWGEOM *lwgeom = lwgeom_from_gserialized(geom);
 
-	POSTGIS_DEBUG(2, "LWGEOM_asEWKT called.");
+	int precision = OUT_DEFAULT_DECIMAL_DIGITS;
+	if (PG_NARGS() > 1)
+		precision = PG_GETARG_INT32(1);
 
-	geom = PG_GETARG_GSERIALIZED_P(0);
-	lwgeom = lwgeom_from_gserialized(geom);
-
-	/* Write to WKT and free the geometry */
-	wkt = lwgeom_to_wkt(lwgeom, WKT_EXTENDED, DBL_DIG, &wkt_size);
-	lwgeom_free(lwgeom);
-
-	/* Write to text and free the WKT */
-	result = cstring_to_text(wkt);
-	lwfree(wkt);
-
-	/* Return the text */
-	PG_FREE_IF_COPY(geom, 0);
-	PG_RETURN_TEXT_P(result);
+	PG_RETURN_TEXT_P(lwgeom_to_wkt_varlena(lwgeom, WKT_EXTENDED, precision));
 }
 
 /**
@@ -2674,8 +2678,7 @@ Datum ST_GeoHash(PG_FUNCTION_ARGS)
 
 	GSERIALIZED *geom = NULL;
 	int precision = 0;
-	char *geohash = NULL;
-	text *result = NULL;
+	lwvarlena_t *geohash = NULL;
 
 	if (PG_ARGISNULL(0))
 	{
@@ -2690,60 +2693,63 @@ Datum ST_GeoHash(PG_FUNCTION_ARGS)
 	}
 
 	geohash = lwgeom_geohash((LWGEOM *)(lwgeom_from_gserialized(geom)), precision);
+	if (geohash)
+		PG_RETURN_TEXT_P(geohash);
+	PG_RETURN_NULL();
+}
 
-	if (!geohash)
+PG_FUNCTION_INFO_V1(_ST_SortableHash);
+Datum _ST_SortableHash(PG_FUNCTION_ARGS)
+{
+	if (PG_ARGISNULL(0))
 		PG_RETURN_NULL();
-
-	result = cstring_to_text(geohash);
-	pfree(geohash);
-
-	PG_RETURN_TEXT_P(result);
+	PG_RETURN_INT64(gserialized_get_sortable_hash(PG_GETARG_GSERIALIZED_P(0)));
 }
 
 PG_FUNCTION_INFO_V1(ST_CollectionExtract);
 Datum ST_CollectionExtract(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *input = PG_GETARG_GSERIALIZED_P(0);
-	GSERIALIZED *output;
-	LWGEOM *lwgeom = lwgeom_from_gserialized(input);
-	LWGEOM *lwcol = NULL;
-	int type = PG_GETARG_INT32(1);
-	int lwgeom_type = lwgeom->type;
+	GSERIALIZED *gser_in, *gser_out;
+	LWGEOM *lwg_in = NULL;
+	LWGEOM *lwg_out = NULL;
+	int extype = 0;
+
+	if (PG_NARGS() > 1)
+		extype = PG_GETARG_INT32(1);
 
 	/* Ensure the right type was input */
-	if (!(type == POINTTYPE || type == LINETYPE || type == POLYGONTYPE))
+	if (!(extype == 0 || extype == POINTTYPE || extype == LINETYPE || extype == POLYGONTYPE))
 	{
-		lwgeom_free(lwgeom);
 		elog(ERROR, "ST_CollectionExtract: only point, linestring and polygon may be extracted");
 		PG_RETURN_NULL();
 	}
 
+	gser_in = PG_GETARG_GSERIALIZED_P(0);
+	lwg_in = lwgeom_from_gserialized(gser_in);
+
 	/* Mirror non-collections right back */
-	if (!lwgeom_is_collection(lwgeom))
+	if (!lwgeom_is_collection(lwg_in))
 	{
 		/* Non-collections of the matching type go back */
-		if (lwgeom_type == type)
+		if (lwg_in->type == extype || !extype)
 		{
-			lwgeom_free(lwgeom);
-			PG_RETURN_POINTER(input);
+			lwgeom_free(lwg_in);
+			PG_RETURN_POINTER(gser_in);
 		}
 		/* Others go back as EMPTY */
 		else
 		{
-			lwcol = lwgeom_construct_empty(
-			    type, lwgeom->srid, lwgeom_has_z(lwgeom), lwgeom_has_m(lwgeom));
+			lwg_out = lwgeom_construct_empty(extype, lwg_in->srid, lwgeom_has_z(lwg_in), lwgeom_has_m(lwg_in));
+			PG_RETURN_POINTER(geometry_serialize(lwg_out));
 		}
 	}
-	else
-	{
-		lwcol = lwcollection_as_lwgeom(lwcollection_extract((LWCOLLECTION *)lwgeom, type));
-	}
 
-	output = geometry_serialize((LWGEOM *)lwcol);
-	lwgeom_free(lwgeom);
-	lwgeom_free(lwcol);
+	lwg_out = (LWGEOM*)lwcollection_extract((LWCOLLECTION*)lwg_in, extype);
 
-	PG_RETURN_POINTER(output);
+	gser_out = geometry_serialize(lwg_out);
+	lwgeom_free(lwg_in);
+	lwgeom_free(lwg_out);
+	PG_RETURN_POINTER(gser_out);
 }
 
 PG_FUNCTION_INFO_V1(ST_CollectionHomogenize);
@@ -2890,49 +2896,56 @@ Datum ST_BoundingDiagonal(PG_FUNCTION_ARGS);
 PG_FUNCTION_INFO_V1(ST_BoundingDiagonal);
 Datum ST_BoundingDiagonal(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *geom_in = PG_GETARG_GSERIALIZED_P(0);
 	GSERIALIZED *geom_out;
 	bool fits = PG_GETARG_BOOL(1);
-	LWGEOM *lwgeom_in = lwgeom_from_gserialized(geom_in);
-	LWGEOM *lwgeom_out;
-	const GBOX *gbox;
-	int hasz = FLAGS_GET_Z(lwgeom_in->flags);
-	int hasm = FLAGS_GET_M(lwgeom_in->flags);
-	int32_t srid = lwgeom_in->srid;
+	LWGEOM *lwgeom_out = NULL;
+
+	GBOX gbox = {0};
+	int hasz;
+	int hasm;
+	int32_t srid;
+
 	POINT4D pt;
 	POINTARRAY *pa;
 
 	if (fits)
 	{
-		/* unregister any cached bbox to ensure it's recomputed */
-		lwgeom_in->bbox = NULL;
-	}
-
-	gbox = lwgeom_get_bbox(lwgeom_in);
-
-	if (!gbox)
-	{
-		lwgeom_out = lwgeom_construct_empty(LINETYPE, srid, hasz, hasm);
+		GSERIALIZED *geom_in = PG_GETARG_GSERIALIZED_P(0);
+		LWGEOM *lwgeom_in = lwgeom_from_gserialized(geom_in);
+		lwgeom_calculate_gbox(lwgeom_in, &gbox);
+		hasz = FLAGS_GET_Z(lwgeom_in->flags);
+		hasm = FLAGS_GET_M(lwgeom_in->flags);
+		srid = lwgeom_in->srid;
 	}
 	else
+	{
+		uint8_t type;
+		lwflags_t flags;
+		int res = gserialized_datum_get_internals_p(PG_GETARG_DATUM(0), &gbox, &flags, &type, &srid);
+		hasz = FLAGS_GET_Z(flags);
+		hasm = FLAGS_GET_M(flags);
+		if (res == LW_FAILURE)
+		{
+			lwgeom_out = lwgeom_construct_empty(LINETYPE, srid, hasz, hasm);
+		}
+	}
+
+	if (!lwgeom_out)
 	{
 		pa = ptarray_construct_empty(hasz, hasm, 2);
-		pt.x = gbox->xmin;
-		pt.y = gbox->ymin;
-		pt.z = gbox->zmin;
-		pt.m = gbox->mmin;
+		pt.x = gbox.xmin;
+		pt.y = gbox.ymin;
+		pt.z = gbox.zmin;
+		pt.m = gbox.mmin;
 		ptarray_append_point(pa, &pt, LW_TRUE);
-		pt.x = gbox->xmax;
-		pt.y = gbox->ymax;
-		pt.z = gbox->zmax;
-		pt.m = gbox->mmax;
+		pt.x = gbox.xmax;
+		pt.y = gbox.ymax;
+		pt.z = gbox.zmax;
+		pt.m = gbox.mmax;
 		ptarray_append_point(pa, &pt, LW_TRUE);
 		lwgeom_out = lwline_as_lwgeom(lwline_construct(srid, NULL, pa));
 	}
 
-	lwgeom_free(lwgeom_in);
-	PG_FREE_IF_COPY(geom_in, 0);
-
 	geom_out = geometry_serialize(lwgeom_out);
 	lwgeom_free(lwgeom_out);
 
