@@ -8,19 +8,12 @@
  */
 #define POSTGIS_PGSQL_VERSION 96
 #define POSTGIS_PGSQL_HR_VERSION 9.6
-#define POSTGIS_GEOS_VERSION 38
+#define POSTGIS_GEOS_VERSION 36
 #define POSTGIS_PROJ_VERSION 72
-#define POSTGIS_LIB_VERSION '3.0.3'
+#define POSTGIS_LIB_VERSION '3.1.1'
 #define POSTGIS_LIBXML2_VERSION 2.9.10
 #define POSTGIS_SFCGAL_VERSION 10308
 
-#if POSTGIS_PGSQL_VERSION >= 96
-#define _PARALLEL PARALLEL SAFE
-#define _PARALLEL_RESTRICTED PARALLEL RESTRICTED
-#else
-#define _PARALLEL
-#define _PARALLEL_RESTRICTED
-#endif
 
 /*
  * High costs can only be used for PostGIS 3/PgSQL 12
@@ -31,8 +24,8 @@
  */
 #if POSTGIS_PGSQL_VERSION >= 120
 #define _COST_DEFAULT COST 1
-#define _COST_LOW COST 100
-#define _COST_MEDIUM COST 1000
+#define _COST_LOW COST 50
+#define _COST_MEDIUM COST 500
 #define _COST_HIGH COST 10000
 #else
 #define _COST_DEFAULT COST 1
@@ -46,14 +39,20 @@
  * (these substitiutions are done with extra quotes sinces CPP
  * won't substitute within apostrophes)
  */
-#define _POSTGIS_SQL_SELECT_POSTGIS_VERSION 'SELECT ''3.0 USE_GEOS=1 USE_PROJ=1 USE_STATS=1''::text AS version'
-#define _POSTGIS_SQL_SELECT_POSTGIS_BUILD_DATE 'SELECT ''2020-11-21 00:48:59''::text AS version'
+#define _POSTGIS_SQL_SELECT_POSTGIS_VERSION 'SELECT ''3.1 USE_GEOS=1 USE_PROJ=1 USE_STATS=1''::text AS version'
+#define _POSTGIS_SQL_SELECT_POSTGIS_BUILD_DATE 'SELECT ''2021-01-28 19:10:50''::text AS version'
 #define _POSTGIS_SQL_SELECT_POSTGIS_PGSQL_VERSION 'SELECT ''96''::text AS version'
 
 #ifdef POSTGIS_REVISION
-#define _POSTGIS_SQL_SELECT_POSTGIS_SCRIPTS_VERSION $$ SELECT trim('3.0.3'::text || $rev$ POSTGIS_REVISION $rev$) AS version $$
+/*
+* Insert the PostGIS revision.  This is immensely goofy because FreeBSD doesn't recognized
+* PostGIS_REVISION as a variable if anything like ` or $rev$ (sql dollar quoting) abutts it so we need to leave a space on both sides
+* which means we need to then trim it to get rid of the extra spaces we added.
+* Maybe someone smarter can come up with a less goofy solution that makes all OS happy
+*/
+#define _POSTGIS_SQL_SELECT_POSTGIS_SCRIPTS_VERSION $$ SELECT trim('3.1.1'::text || $rev$ POSTGIS_REVISION $rev$) AS version $$
 #else
-#define _POSTGIS_SQL_SELECT_POSTGIS_SCRIPTS_VERSION $$ SELECT '3.0.3'::text AS version $$
+#define _POSTGIS_SQL_SELECT_POSTGIS_SCRIPTS_VERSION $$ SELECT '3.1.1'::text AS version $$
 #endif
 
 #define SRID_USR_MAX 998999
