@@ -160,11 +160,7 @@ select '125',a &>b from TEST;
 select '126',a ~= b from TEST;
 select '127',a @ b from TEST;
 select '128',a ~ b from TEST;
-
--- ST_Mem_Size was deprecated in favor of ST_MemSize in 2.2.0
---  ST_Mem_Size will be removed in 2.4.0
 select '129', ST_MemSize(PostGIS_DropBBOX(a)), ST_MemSize(PostGIS_DropBBOX(b)) from TEST;
-
 select '131', ST_X('POINT(1 2)');
 select '132', ST_Y('POINT(1 2)');
 select '133', ST_Z('POINT(1 2)');
@@ -265,16 +261,64 @@ select '225', ST_Expand('BOX(-2 3, -1 6'::BOX2D, 4, 2);
 select '226', ST_SRID(ST_Expand('SRID=4326;POINT (0 0)'::geometry, 1))=4326;
 
 -- ST_TileEnvelope()
-select '227', ST_AsEWKT(ST_TileEnvelope(-1, 0, 0));
-select '228', ST_AsEWKT(ST_TileEnvelope(0, 0, 1));
-select '229', ST_AsEWKT(ST_TileEnvelope(0, 0, 0));
-select '230', ST_AsText(ST_TileEnvelope(4, 8, 8), 7);
-select '231', ST_AsEWKT(ST_TileEnvelope(4, 15, 15));
-select '232', ST_AsEWKT(ST_TileEnvelope(4, 8, 8, ST_MakeEnvelope(-100, -100, 100, 100, 0)));
-select '233', ST_AsEWKT(ST_TileEnvelope(4, 15, 15, ST_MakeEnvelope(-100, -100, 100, 100, 0)));
-select '234', ST_AsEWKT(ST_TileEnvelope(4, 0, 0, ST_MakeEnvelope(-100, -100, 100, 100, 0)));
-select '235', ST_AsEWKT(ST_TileEnvelope(4, 8, 8, ST_MakeEnvelope(-200, -100, 200, 100, 0)));
+select '227', ST_AsText(ST_TileEnvelope(-1, 0, 0), 2);
+select '228', ST_AsText(ST_TileEnvelope(0, 0, 1), 2);
+select '229', ST_AsText(ST_TileEnvelope(0, 0, 0), 2);
+select '230', ST_AsText(ST_TileEnvelope(4, 8, 8), 2);
+select '231', ST_AsText(ST_TileEnvelope(4, 15, 15), 2);
+select '232', ST_AsText(ST_TileEnvelope(4, 8, 8, ST_MakeEnvelope(-100, -100, 100, 100, 0)), 2);
+select '233', ST_AsText(ST_TileEnvelope(4, 15, 15, ST_MakeEnvelope(-100, -100, 100, 100, 0)), 2);
+select '234', ST_AsText(ST_TileEnvelope(4, 0, 0, ST_MakeEnvelope(-100, -100, 100, 100, 0)), 2);
+select '235', ST_AsText(ST_TileEnvelope(4, 8, 8, ST_MakeEnvelope(-200, -100, 200, 100, 0)), 2);
+select '236', ST_AsText(ST_TileEnvelope(0, 0, 0, margin => 0.1), 2);
+select '237', ST_AsText(ST_TileEnvelope(1, 0, 0, margin => 0.1), 2);
+select '238', ST_AsText(ST_TileEnvelope(2, 1, 3, margin => 0.5), 2);
+select '239', ST_AsText(ST_TileEnvelope(0, 0, 0, margin => -0.5), 2);
+select '240', ST_AsText(ST_TileEnvelope(0, 0, 0, margin => -0.51), 2);
+select '241', ST_AsText(ST_TileEnvelope(0, 0, 0, margin => -0.4), 2);
+select '250', ST_AsText(ST_TileEnvelope(10,300,387), 2);
+select '251', ST_AsText(ST_TileEnvelope(10,300,387, margin => 0.1), 2);
+select '252', ST_AsText(ST_TileEnvelope(10,300,387, margin => 0.5), 2);
+select '253', ST_AsText(ST_TileEnvelope(10,300,387, margin => 2), 2);
+select '254', ST_AsText(ST_TileEnvelope(10,300,387, margin => -0.3), 2);
 
+-- ST_Hexagon()
+select '300', ST_AsText(ST_Hexagon(10, 0, 0), 5);
+select '301', ST_AsText(ST_Hexagon(10, 1, 1), 5);
+select '302', ST_AsText(ST_Hexagon(10, -1, -1), 5);
+select '303', ST_AsText(ST_Hexagon(10, 100, -100), 5);
+-- ST_Square()
+select '304', ST_AsText(ST_Square(10, 0, 0), 5);
+select '305', ST_AsText(ST_Square(10, 1, 1), 5);
+select '306', ST_AsText(ST_Square(10, -1, -1), 5);
+select '307', ST_AsText(ST_Square(10, 100, -100), 5);
+-- ST_HexagonGrid()
+select '308', Count(*) FROM ST_HexagonGrid(100000, ST_TileEnvelope(4, 7, 7));
+select '309', Count(*) FROM ST_HexagonGrid(100000, ST_TileEnvelope(4, 7, 7)) hex, ST_TileEnvelope(4, 7, 7) tile WHERE NOT ST_Intersects(hex.geom, tile);
+select '310', Count(*) FROM ST_HexagonGrid(100000, ST_TileEnvelope(4, 2, 2));
+select '311', Count(*) FROM ST_HexagonGrid(100000, ST_TileEnvelope(4, 2, 2)) hex, ST_TileEnvelope(4, 2, 2) tile WHERE NOT ST_Intersects(hex.geom, tile);
+-- ST_SquareGrid()
+select '312', Count(*) FROM ST_SquareGrid(100000, ST_TileEnvelope(4, 7, 7));
+select '313', Count(*) FROM ST_SquareGrid(100000, ST_TileEnvelope(4, 7, 7)) hex, ST_TileEnvelope(4, 7, 7) tile WHERE NOT ST_Intersects(hex.geom, tile);
+select '314', Count(*) FROM ST_SquareGrid(100000, ST_TileEnvelope(4, 2, 2));
+select '315', Count(*) FROM ST_SquareGrid(100000, ST_TileEnvelope(4, 2, 2)) hex, ST_TileEnvelope(4, 2, 2) tile WHERE NOT ST_Intersects(hex.geom, tile);
+
+WITH geoms AS
+(
+    SELECT * FROM  (SELECT (ST_HexagonGrid(5, '01030000000100000005000000fdffffffff7f66c0fcffffffff7f56c0fdffffffff7f66c07cc0e71a95e8544000000000008066407cc0e71a95e854400000000000806640fcffffffff7f56c0fdffffffff7f66c0fcffffffff7f56c0')).*) sq  WHERE i = 5 AND j IN (0, 1)
+),
+j0 AS
+(
+    SELECT * FROM geoms WHERE i = 5 AND j = 0
+),
+j1 AS
+(
+    SELECT * FROM geoms WHERE i = 5 AND j = 1
+)
+SELECT '316',
+    ST_Intersects(j0.geom, j1.geom),
+    ST_AsText(ST_Intersection(j0.geom, j1.geom))
+FROM j0, j1;
 
 -- Drop test table
 DROP table test;
@@ -285,9 +329,12 @@ DROP table test;
 SELECT DISTINCT 'unexpected probin', proname || ':' || probin
 FROM pg_proc
 WHERE probin like '%postgis%'
-   AND regexp_replace(probin, '(rt)?postgis(_[^-]*)?', '') !=
+  AND
+regexp_replace(probin, '(rt)?postgis(_[^-]*)?(-[0-9.]*)$', '\3')
+	!=
 (
-SELECT regexp_replace(probin, '(rt)?postgis(_[^-]*)?', '')
+	SELECT
+regexp_replace(probin, '(rt)?postgis(_[^-]*)?(-[0-9.]*)$', '\3')
 	FROM pg_proc WHERE proname = 'postgis_lib_version'
 )
 ORDER BY 2;
