@@ -1,4 +1,4 @@
-#!/usr/bin/perl
+#!/usr/bin/env perl
 
 #
 # PostGIS - Spatial Types for PostgreSQL
@@ -15,6 +15,7 @@
 #$| = 1;
 use File::Basename;
 use File::Temp 'tempdir';
+use Time::HiRes qw(time);
 #use File::Which;
 use File::Copy;
 use File::Path;
@@ -69,8 +70,8 @@ my $OPT_EXTVERSION = '';
 my $OPT_UPGRADE_PATH = '';
 our $OPT_UPGRADE_FROM = '';
 my $OPT_UPGRADE_TO = '';
-my $VERBOSE = 0;
-my $OPT_SCHEMA = 'public';
+our $VERBOSE = 0;
+our $OPT_SCHEMA = 'public';
 
 GetOptions (
 	'verbose' => \$VERBOSE,
@@ -243,6 +244,7 @@ else
 	}
 }
 
+my $pgvernum = sql("SELECT current_setting('server_version_num')");
 my $libver = sql("select postgis_lib_version()");
 my $defextver = sql("select default_version from pg_available_extensions where name = 'postgis'");
 
@@ -311,14 +313,25 @@ sub create_upgrade_test_objects
     exit(1);
   }
 
-  # my $query = "create view upgrade_view_test as ";
-  # $query .= "select st_union(g1) from upgrade_test;";
-  # my $ret = sql($query);
-  # unless ( $ret =~ /^CREATE/ ) {
-  #   `dropdb $DB`;
-  #   print "\nSomething went wrong creating upgrade_view_test view: $ret.\n";
-  #   exit(1);
-  # }
+
+  if ( $pgvernum >= 120000 ) {
+    # We know upgrading with an st_union() based view
+    # fails unless you're on PostgreSQL 12, so we don't
+    # even try that.
+    #
+    # We could re-enable this test IF we fix the upgrade
+    # in pre-12 versions. Refer to
+    # https://trac.osgeo.org/postgis/ticket/4386
+    #
+    $query = "create view upgrade_view_test as ";
+    $query .= "select st_union(g1) from upgrade_test;";
+    my $ret = sql($query);
+    unless ( $ret =~ /^CREATE/ ) {
+      `dropdb $DB`;
+      print "\nSomething went wrong creating upgrade_view_test view: $ret.\n";
+      exit(1);
+    }
+  }
 
   if ( $OPT_WITH_RASTER )
   {
@@ -356,12 +369,12 @@ sub drop_upgrade_test_objects
 {
   # TODO: allow passing the "upgrade-cleanup" script via commandline
 
-  # my $ret = sql("drop view upgrade_view_test;");
-  # unless ( $ret =~ /^DROP/ ) {
-  #   `dropdb $DB`;
-  #   print "\nSomething went wrong dropping spatial view: $ret.\n";
-  #   exit(1);
-  # }
+  my $ret = sql("drop view if exists upgrade_view_test;");
+  unless ( $ret =~ /^DROP/ ) {
+    `dropdb $DB`;
+    print "\nSomething went wrong dropping spatial view: $ret.\n";
+    exit(1);
+  }
 
   my $ret = sql("drop table upgrade_test;");
   unless ( $ret =~ /^DROP/ ) {
@@ -385,6 +398,8 @@ sub drop_upgrade_test_objects
 
 if ( $OPT_UPGRADE )
 {
+	print "  Upgrading from postgis $libver\n";
+
   create_upgrade_test_objects();
 
   if ( $OPT_EXTENSIONS )
@@ -414,17 +429,20 @@ if ( $OPT_DUMPRESTORE )
 
 my $geosver =  sql("select postgis_geos_version()");
 my $projver = sql("select postgis_proj_version()");
-my $svnrev = sql("select postgis_svn_version()");
 my $libbuilddate = sql("select postgis_lib_build_date()");
 my $pgsqlver = sql("select version()");
 my $gdalver = sql("select postgis_gdal_version()") if $OPT_WITH_RASTER;
 my $sfcgalver = sql("select postgis_sfcgal_version()") if $OPT_WITH_SFCGAL;
 my $scriptver = sql("select postgis_scripts_installed()");
+# postgis_lib_revision was introduced in 3.1.0
+my $librev = semver_lessthan($scriptver, "3.1.0") ?
+							sql("select postgis_svn_version()") :
+							sql("select postgis_lib_revision()");
 my $raster_scriptver = sql("select postgis_raster_scripts_installed()")
   if ( $OPT_WITH_RASTER );
 
 print "$pgsqlver\n";
-print "  Postgis $libver - r${svnrev} - $libbuilddate\n";
+print "  Postgis $libver - (${librev}) - $libbuilddate\n";
 print "  scripts ${scriptver}\n";
 print "  raster scripts ${raster_scriptver}\n" if ( $OPT_WITH_RASTER );
 print "  GEOS: $geosver\n" if $geosver;
@@ -451,6 +469,7 @@ foreach $TEST (@ARGV)
 {
 	my $TEST_OBJ_COUNT_PRE;
 	my $TEST_OBJ_COUNT_POST;
+	my $TEST_START_TIME;
 
 	# catch a common mistake (strip trailing .sql)
 	$TEST =~ s/.sql$//;
@@ -459,7 +478,12 @@ foreach $TEST (@ARGV)
 	$TEST_OBJ_COUNT_PRE = count_postgis_objects();
 
 	# Check for a "-pre.pl" file in case there are setup commands
-    eval_file("${TEST}-pre.pl");
+	unless ( eval_file("${TEST}-pre.pl") )
+	{
+		chop($@);
+		fail("Failed evaluating ${TEST}-pre.pl: $@");
+		next;
+	}
 
 	# Check for a "-pre.sql" file in case there is setup SQL needed before
 	# the test can be run.
@@ -473,23 +497,24 @@ foreach $TEST (@ARGV)
 	# create the .sql
 	# Check for .dbf not just .shp since the loader can load
 	# .dbf files without a .shp.
+	$TEST_START_TIME = time;
 	if ( -r "${TEST}.dbf" )
 	{
-		pass() if ( run_loader_test() );
+		pass("in ".int(1000*(time-$TEST_START_TIME))." ms") if ( run_loader_test() );
 	}
 	elsif ( -r "${TEST}.tif" )
 	{
 		my $rv = run_raster_loader_test();
-		pass() if $rv;
+		pass("in ".int(1000*(time-$TEST_START_TIME))." ms") if $rv;
 	}
 	elsif ( -r "${TEST}.sql" )
 	{
 		my $rv = run_simple_test("${TEST}.sql", "${TEST}_expected");
-		pass() if $rv;
+		pass("in ".int(1000*(time-$TEST_START_TIME))." ms") if $rv;
 	}
 	elsif ( -r "${TEST}.dmp" )
 	{
-		pass() if run_dumper_test();
+		pass("in ".int(1000*(time-$TEST_START_TIME))." ms") if run_dumper_test();
 	}
 	else
 	{
@@ -509,14 +534,13 @@ foreach $TEST (@ARGV)
 	}
 
 	# Check for a "-post.pl" file in case there are teardown commands
-    eval_file("${TEST}-post.pl");
+	eval_file("${TEST}-post.pl");
 
 	$TEST_OBJ_COUNT_POST = count_postgis_objects();
 
 	if ( $TEST_OBJ_COUNT_POST != $TEST_OBJ_COUNT_PRE )
 	{
 		fail("PostGIS object count pre-test ($TEST_OBJ_COUNT_POST) != post-test ($TEST_OBJ_COUNT_PRE)");
-		return 0;
 	}
 
 }
@@ -550,7 +574,6 @@ if ( $OPT_CLEAN )
 
 if ( ! ($OPT_NODROP || $OPT_NOCREATE) )
 {
-	sleep(1);
 	system("dropdb $DB");
 }
 else
@@ -588,6 +611,8 @@ Options:
                        if available.
   --dumprestore   dump and restore spatially-enabled db before running tests
   --nodrop        do not drop the regression database on exit
+  --schema        where to install/find PostGIS (relocatable) PostGIS
+                  (defaults to "public")
   --raster        load also raster extension
   --topology      load also topology extension
   --sfcgal        use also sfcgal backend
@@ -623,8 +648,8 @@ sub show_progress
 # pass <msg>
 sub pass
 {
-  my $msg = shift;
-  printf(" ok %s\n", $msg);
+    my $msg = shift;
+    printf(" ok %s\n", $msg);
 }
 
 # fail <msg> <log>
@@ -710,7 +735,7 @@ sub drop_table
 sub sql
 {
 	my $sql = shift;
-	my $result = `psql -tXA -d $DB -c "$sql"`;
+	my $result = `psql -tXA -d $DB -c 'SET search_path TO public,$OPT_SCHEMA' -c "$sql" | sed '/^SET\$/d'`;
 	$result =~ s/[\n\r]*$//;
 	$result;
 }
@@ -721,12 +746,10 @@ sub eval_file
     my $pl;
     if ( -r $file )
     {
-        #open(PL, $file);
-        #$pl = <PL>;
-        #close(PL);
-        #eval($pl);
-				do $file;
+				do $file or return 0;
+				#system($^X, $file) == 0 or return 0;
     }
+		1;
 }
 
 ##################################################################
@@ -766,12 +789,15 @@ sub run_simple_test
 	chmod 0777, $betmpdir;
 
 	my $scriptdir = scriptdir($libver, $OPT_EXTENSIONS);
-	my $cmd = "psql -v \"VERBOSITY=terse\""
+
+	my ($sqlfile,$sqldir) = fileparse($sql);
+	my $cmd = "cd $sqldir; psql -v \"VERBOSITY=terse\""
           . " -v \"tmpfile='$tmpfile'\""
           . " -v \"scriptdir=$scriptdir\""
           . " -v \"regdir=$REGDIR\""
           . " -v \"schema=$OPT_SCHEMA.\""
-          . " -tXAq $DB < $sql > $outfile 2>&1";
+          . " -c \"SET search_path TO public,$OPT_SCHEMA,topology\""
+          . " -tXAq -f $sqlfile $DB > $outfile 2>&1";
 	my $rv = system($cmd);
 
 	# Check for ERROR lines
@@ -1067,10 +1093,14 @@ sub run_loader_test
 	if ( -r $opts_file )
 	{
 		open(FILE, $opts_file);
-		my @opts = <FILE>;
+		my @opts;
+		while (<FILE>) {
+			next if /^\s*#/;
+			chop;
+			s/{regdir}/$REGDIR/;
+			push @opts, $_;
+		}
 		close(FILE);
-		@opts = grep(!/^\s*#/, @opts);
-		map(s/\n//, @opts);
 		$custom_opts = join(" ", @opts);
 	}
 
@@ -1141,9 +1171,9 @@ sub run_loader_test
 #  Run dumper and compare output with various expectances
 #  test and run simple test with provided expected output.
 #
-# input is ${TEST}.dmp, where last line is considered to be the
-# [table|query] argument for pgsql2shp and all the previous lines,
-# if any are
+# input is ${TEST}.dmp, where first line is considered to be the
+# [table|query] argument for pgsql2shp and all the next lines,
+# if any are options.
 #
 ##################################################################
 sub run_dumper_test
@@ -1159,12 +1189,16 @@ sub run_dumper_test
 
   # Produce the output SHP file.
   open DUMPFILE, "$dump_file" or die "Cannot open dump file $dump_file\n";
-  sleep(1);
   my @dumplines = <DUMPFILE>;
   close DUMPFILE;
-  my $dumpstring = join '', @dumplines;
-  chop($dumpstring);
-  my @cmd = ("${PGSQL2SHP}", "-f", ${shpfile}, ${DB}, ${dumpstring});
+  chomp(@dumplines);
+  my $dumpstring = shift @dumplines;
+  @dumplines = map { local $_ = $_; s/{regdir}/$REGDIR/; $_ } @dumplines;
+  my @cmd = ("${PGSQL2SHP}", "-f", ${shpfile});
+  push @cmd, @dumplines;
+  push @cmd, ${DB};
+  push @cmd, $dumpstring;
+  #print "CMD: " . join (' ', @cmd) . "\n";
   open my $stdout_save, '>&', *STDOUT or die "Cannot dup stdout\n";
   open my $stderr_save, '>&', *STDERR or die "Cannot dup stdout\n";
   open STDOUT, ">${outfile}" or die "Cannot write to ${outfile}\n";
@@ -1172,7 +1206,6 @@ sub run_dumper_test
   my $rv = system(@cmd);
   open STDERR, '>&', $stderr_save;
   open STDOUT, '>&', $stdout_save;
-  #sleep(3);
   show_progress();
 
   if ( $rv )
@@ -1265,7 +1298,9 @@ sub count_db_objects
 		select count(*) from pg_operator union all
 		select count(*) from pg_opclass union all
 		select count(*) from pg_namespace
-			where nspname NOT LIKE 'pg_%' union all
+			where nspname NOT LIKE 'pg_%'
+			  and nspname != '${OPT_SCHEMA}'
+		union all
 		select count(*) from pg_opfamily
 		)
 		select sum(count) from counts");
@@ -1333,8 +1368,10 @@ sub load_sql_file
 	{
 		# ON_ERROR_STOP is used by psql to return non-0 on an error
 		my $psql_opts = "--no-psqlrc --variable ON_ERROR_STOP=true";
-		my $cmd = "psql $psql_opts -Xf $file $DB >> $REGRESS_LOG 2>&1";
-		print "  $file\n" if $VERBOSE;
+		my $cmd = "psql $psql_opts -c 'CREATE SCHEMA IF NOT EXISTS $OPT_SCHEMA' ";
+		$cmd .= "-c 'SET search_path TO $OPT_SCHEMA,topology'";
+		$cmd .= " -Xf $file $DB >> $REGRESS_LOG 2>&1";
+		#print "  $file\n" if $VERBOSE;
 		my $rv = system($cmd);
 		if ( $rv )
 		{
@@ -1350,6 +1387,15 @@ sub prepare_spatial_extensions
 {
 	# ON_ERROR_STOP is used by psql to return non-0 on an error
 	my $psql_opts = "--no-psqlrc --variable ON_ERROR_STOP=true";
+
+	my $sql = "CREATE SCHEMA IF NOT EXISTS ${OPT_SCHEMA}";
+	my $cmd = "psql $psql_opts -c \"". $sql . "\" $DB >> $REGRESS_LOG 2>&1";
+	my $rv = system($cmd);
+	if ( $rv ) {
+	  fail "Error encountered creating target schema ${OPT_SCHEMA}", $REGRESS_LOG;
+	  die;
+	}
+
 	my $sql = "CREATE EXTENSION postgis";
 
 	if ( $OPT_UPGRADE_FROM ) {
@@ -1360,6 +1406,8 @@ sub prepare_spatial_extensions
 		$sql .= " VERSION '" . $OPT_UPGRADE_FROM . "'";
 	}
 
+	$sql .= " SCHEMA " . $OPT_SCHEMA;
+
 	print "Preparing db '${DB}' using: ${sql}\n";
 
 	my $cmd = "psql $psql_opts -c \"". $sql . "\" $DB >> $REGRESS_LOG 2>&1";
@@ -1399,6 +1447,8 @@ sub prepare_spatial_extensions
 			$sql .= " VERSION '" . $OPT_UPGRADE_FROM . "'";
 		}
 
+		$sql .= " SCHEMA " . $OPT_SCHEMA;
+
 		print "Preparing db '${DB}' using: ${sql}\n";
 
  		$cmd = "psql $psql_opts -c \"" . $sql . "\" $DB >> $REGRESS_LOG 2>&1";
@@ -1423,6 +1473,8 @@ sub prepare_spatial_extensions
 				$sql .= " VERSION '" . $OPT_UPGRADE_FROM . "'";
 			}
 
+			$sql .= " SCHEMA " . $OPT_SCHEMA;
+
 			print "Preparing db '${DB}' using: ${sql}\n";
 
 			$cmd = "psql $psql_opts -c \"" . $sql . "\" $DB >> $REGRESS_LOG 2>&1";
@@ -1449,7 +1501,6 @@ sub prepare_spatial
 	# Load postgis.sql into the database
 	load_sql_file("${scriptdir}/postgis.sql", 1);
 	load_sql_file("${scriptdir}/postgis_comments.sql", 0);
-	load_sql_file("${scriptdir}/postgis_proc_set_search_path.sql", 0);
 	load_sql_file("${scriptdir}/spatial_ref_sys.sql", 0);
 
 	if ( $OPT_WITH_TOPO )
@@ -1464,7 +1515,6 @@ sub prepare_spatial
 		print "Loading Raster into '${DB}'\n";
 		load_sql_file("${scriptdir}/rtpostgis.sql", 1);
 		load_sql_file("${scriptdir}/raster_comments.sql", 0);
-		load_sql_file("${scriptdir}/rtpostgis_proc_set_search_path.sql", 0);
 	}
 
 	if ( $OPT_WITH_SFCGAL )
@@ -1477,6 +1527,20 @@ sub prepare_spatial
 	return 1;
 }
 
+sub package_extension_sql
+{
+	my ($extname, $extver) = @_;
+	my $sql;
+
+	if ( $pgvernum lt 130000 ) {
+		$sql = "CREATE EXTENSION ${extname} VERSION '${extver}' FROM unpackaged;";
+	} else {
+		$sql = "CREATE EXTENSION ${extname} VERSION unpackaged;";
+		$sql .= "ALTER EXTENSION ${extname} UPDATE TO '${extver}'";
+	}
+	return $sql;
+}
+
 # Upgrade an existing database (soft upgrade)
 sub upgrade_spatial
 {
@@ -1581,7 +1645,7 @@ sub upgrade_spatial_extensions
     }
     elsif ( $OPT_UPGRADE_FROM =~ /^unpackaged/ )
     {
-      $sql = "CREATE EXTENSION postgis VERSION '${nextver}' FROM unpackaged";
+			$sql = package_extension_sql('postgis', ${nextver});
     }
     else
     {
@@ -1594,7 +1658,7 @@ sub upgrade_spatial_extensions
     #print "CMD: " . $cmd . "\n";
     my $rv = system($cmd);
     if ( $rv ) {
-      fail "Error encountered altering EXTENSION POSTGIS", $REGRESS_LOG;
+      fail "Error encountered updating EXTENSION POSTGIS", $REGRESS_LOG;
       die;
     }
 
@@ -1607,7 +1671,7 @@ sub upgrade_spatial_extensions
       {
         # upgrade of postgis must have unpackaged raster, so
         # we create it again here
-        my $sql = "CREATE EXTENSION postgis_raster VERSION '${nextver}' FROM unpackaged";
+				my $sql = package_extension_sql('postgis_raster', ${nextver});
 
         print "Upgrading PostGIS Raster in '${DB}' using: ${sql}\n" ;
 
@@ -1624,7 +1688,8 @@ sub upgrade_spatial_extensions
         # left unpackaged
         print "Packaging PostGIS Raster in '${DB}' for later drop using: ${sql}\n" ;
 
-        $sql = "CREATE EXTENSION postgis_raster VERSION '${nextver}' FROM unpackaged";
+				$sql = package_extension_sql('postgis_raster', ${nextver});
+
         $cmd = "psql $psql_opts -c \"" . $sql . "\" $DB >> $REGRESS_LOG 2>&1";
         $rv = system($cmd);
         if ( $rv ) {
@@ -1655,7 +1720,7 @@ sub upgrade_spatial_extensions
         my $sql = "ALTER EXTENSION postgis_raster UPDATE TO '${nextver}'";
 
         if ( $OPT_UPGRADE_FROM =~ /^unpackaged/ ) {
-          $sql = "CREATE EXTENSION postgis_raster VERSION '${nextver}' FROM unpackaged";
+					$sql = package_extension_sql('postgis_raster', ${nextver});
         }
 
         print "Upgrading PostGIS Raster in '${DB}' using: ${sql}\n" ;
@@ -1663,7 +1728,7 @@ sub upgrade_spatial_extensions
         my $cmd = "psql $psql_opts -c \"" . $sql . "\" $DB >> $REGRESS_LOG 2>&1";
         my $rv = system($cmd);
         if ( $rv ) {
-          fail "Error encountered altering EXTENSION POSTGIS_RASTER", $REGRESS_LOG;
+          fail "Error encountered updating EXTENSION POSTGIS_RASTER", $REGRESS_LOG;
           die;
         }
     }
@@ -1673,7 +1738,7 @@ sub upgrade_spatial_extensions
       my $sql = "ALTER EXTENSION postgis_topology UPDATE TO '${nextver}'";
 
 			if ( $OPT_UPGRADE_FROM =~ /^unpackaged/ ) {
-				$sql = "CREATE EXTENSION postgis_topology VERSION '${nextver}' FROM unpackaged";
+				$sql = package_extension_sql('postgis_topology', ${nextver});
 			}
 
       print "Upgrading PostGIS Topology in '${DB}' using: ${sql}\n";
@@ -1681,7 +1746,7 @@ sub upgrade_spatial_extensions
       my $cmd = "psql $psql_opts -c \"" . $sql . "\" $DB >> $REGRESS_LOG 2>&1";
       my $rv = system($cmd);
       if ( $rv ) {
-        fail "Error encountered altering EXTENSION POSTGIS_TOPOLOGY", $REGRESS_LOG;
+        fail "Error encountered updating EXTENSION POSTGIS_TOPOLOGY", $REGRESS_LOG;
         die;
       }
     }
@@ -1691,7 +1756,7 @@ sub upgrade_spatial_extensions
 			my $sql;
 
 			if ( $OPT_UPGRADE_FROM =~ /^unpackaged/ ) {
-				$sql = "CREATE EXTENSION postgis_sfcgal VERSION '${nextver}' FROM unpackaged";
+				$sql = package_extension_sql('postgis_sfcgal', ${nextver});
 			}
 			elsif ( $OPT_UPGRADE_FROM && semver_lessthan($OPT_UPGRADE_FROM, "2.2.0") )
 			{
