@@ -3,7 +3,7 @@
 set client_min_messages to WARNING;
 
 -- Import city_data
-\i load_topology-4326.sql
+\i ../load_topology-4326.sql
 
 -- Save max node id
 select 'node'::text as what, max(node_id) INTO city_data.limits FROM city_data.node;
